//! # Post API entry
//!
//! @author Pierre Hubert
use serde::Serialize;

use crate::data::post::{Post, PostKind};
use crate::data::user::UserID;
use crate::utils::user_data_utils::user_data_url;

#[derive(Serialize)]
#[allow(non_snake_case)]
pub struct PostAPI {
    ID: u64,
    userID: u64,
    user_page_id: u64,
    group_id: u64,
    post_time: u64,
    content: Option<String>,
    visibility_level: String,
    kind: String,

    // File specific
    file_size: Option<usize>,
    file_type: Option<String>,
    file_path: Option<String>,
    file_path_url: Option<String>,

    // Weblink specific
    link_url: Option<String>,
    link_title: Option<String>,
    link_description: Option<String>,
    link_image: Option<String>,
}

impl PostAPI {
    /// Turn a `Post` entry into an API entry
    pub fn new(p: &Post) -> PostAPI {
        let mut post = PostAPI {
            ID: p.id,
            userID: p.user_id.id(),
            user_page_id: p.user_page_id().unwrap_or(&UserID::invalid()).id(),
            group_id: p.group_id().map(|f| f.id()).unwrap_or(0),
            post_time: p.time_create,
            content: p.content.clone(),
            visibility_level: p.visibility.to_api(),
            kind: p.kind.to_api(),

            // File specific
            file_size: None,
            file_type: None,
            file_path: None,
            file_path_url: None,

            // Weblink specific
            link_url: None,
            link_title: None,
            link_description: None,
            link_image: None,
        };

        match &p.kind {
            PostKind::POST_KIND_TEXT => { /* do nothing */ }

            PostKind::POST_KIND_IMAGE(file) | PostKind::POST_KIND_PDF(file) => {
                post.file_size = Option::from(file.size);
                post.file_type = file.file_type.clone();
                post.file_path = Some(file.path.clone());
                post.file_path_url = Some(user_data_url(file.path.as_ref()))
            }

            PostKind::POST_KIND_WEBLINK(link) => {
                post.link_url = Some(link.url.clone());
                post.link_description = link.description.clone();
                post.link_title = link.title.clone();
                post.link_image = link.image.clone();
            }

            PostKind::POST_KIND_MOVIE => {}
            PostKind::POST_KIND_COUNTDOWN => {}
            PostKind::POST_KIND_SURVEY => {}
            PostKind::POST_KIND_YOUTUBE => {}
        }

        post
    }

    /// Turn a list of posts into an API entry
    pub fn for_list(l: &Vec<Post>) -> Vec<PostAPI> {
        l.iter().map(Self::new).collect()
    }
}